@@ -859,7 +859,9 @@ sub DeleteHeader {
   my $metadata = $message->{metadata};
   for ($hdrnum=0; $hdrnum<@{$metadata->{headers}}; $hdrnum++) {
     # Skip if they are using a header name and it doesn't match
-    next if !$usingregexp && lc $metadata->{headers}[$hdrnum]{name} ne lc $key;
+    # Quotemeta the header name we are checking as we have done it to $key.
+    next if !$usingregexp &&
+            lc(quotemeta($metadata->{headers}[$hdrnum]{name})) ne lc $key;
     # Skip if they are using a regexp and it doesn't match
     $line = $metadata->{headers}[$hdrnum]{name} . $metadata->{headers}[$hdrnum]{body};
     next if $usingregexp && $line !~ /$key/i;
@@ -1654,7 +1656,8 @@ sub CreateBatch {
       $newmessage->{mtime} = $mtime;
       $batchempty = 0;
 
-      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+      if (MailScanner::Config::Value("scanmail", $newmessage) =~ /[12]/ ||
+          MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
           MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
 	$newmessage->NeedsScanning(1);
 	$DirtyMsgs++;
