@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm 5028 2010-06-09 21:21:29Z sysjkf $
+#   $Id: MessageBatch.pm 5048 2010-08-03 11:19:15Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ use DBI qw(:sql_types);
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 5028 $, 10;
+$VERSION = substr q$Revision: 5048 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -260,6 +260,7 @@ sub SpamChecks {
   while(($id, $message) = each %{$this->{messages}}) {
     next if !$message->{scanmail};
     next if $message->{deleted};
+    next if $message->{scanvirusonly}; # Over-rides Spam Checks setting
     next unless MailScanner::Config::Value('spamchecks', $message) =~ /1/;
  
     #print STDERR "Spam checks for $id\n";
@@ -440,7 +441,11 @@ sub Empty {
 
   my($id, $message);
   while(($id,$message) = each %{$this->{messages}}) {
-    return 0 unless $message->{deleted};
+    if (!$message->{deleted}) {
+      # Do not remove the next line, it is vital to reset "each()"!
+      keys %{$this->{messages}};
+      return 0;
+    }
   }
   return 1;
 }
@@ -975,7 +980,11 @@ sub WarnLocalPostmaster {
     next if $message->{deleted} && !$message->{stillwarn};
     next unless MailScanner::Config::Value('sendnotices', $message) =~ /1/;
     $posties = MailScanner::Config::Value('noticerecipient', $message);
-    return if $posties =~ /^\s*$/; # Return if no opsties defined
+    if ($posties =~ /^\s*$/) {
+      keys %{$this->{messages}}; # Necessary line to reset "each()"
+      # Return if no posties defined
+      return;
+    }
     @posties = split(" ", $posties);
     foreach $postie (@posties) {
       $headers{$postie} = $message->CreatePostmasterHeaders($postie)
