@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm 4824 2009-05-15 09:26:50Z sysjkf $
+#   $Id: Postfix.pm 5044 2010-07-31 16:58:58Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ use Encode;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4824 $, 10;
+$VERSION = substr q$Revision: 5044 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -417,11 +417,10 @@ sub new {
         $pRecordsFound = 1; # If we find a p record we need remember this to handle body ...
 	next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	if ($recdata+0 > $MaxpRecPos) {
-	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	    $ErrorFound = 1;
 	    last;
 	}
-	push @npos, $recdata+0; # save jumpto position, for loop detection.
 	if ($OriginalPos == 0) {
 	   # Jump to after E record and commence
 	   $OriginalPos = tell $RQf;
@@ -430,31 +429,29 @@ sub new {
 	} else {
 	  # We're at the return point, or moving even furtehr away...
 	  if ($recdata+0 < $OriginalPos) {
-	     # Error exit, carp or whatever. Probably should do the
-	     # nice exit Jules does above. For now, warn and use
-	     # OriginalPos
-	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	     seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	     $OriginalPos = 0; # We're home again... Reset to prepare for
-			       # any new warps
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	     #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0;
 	     @npos = ();
+	     $ErrorFound = 1;
+	     last;
 	  } else {
 	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-            if ($recdata+0 < $MsgContSize+$DataOffset) {
-	    #20090216 if ($recdata+0 < $npos[$#npos-1]) {
-	       if ($recdata+0 >= $npos[0]) {
-		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+            foreach $_ (@npos) {
+              if ($_+0 eq $recdata+0) {
 		  $ErrorFound = 1;
-		  $OriginalPos = 0; # Reset to not fool next segment loop
-	          @npos = ();
 		  last;
 	       }
-	       $OriginalPos = 0; # We're home again... Reset to prepare for
-		                 # any new warps
+	    }
+            if ($ErrorFound) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $OriginalPos = 0; # Reset to not fool next segment loop
 	       @npos = ();
+		  last;
 	    }
 	  }
 	}
+	push @npos, $recdata+0; # save jumpto position, for loop detection.
       } else {
         # Some other record type. Just store it and move on.
         push @{$message->{metadata}}, "$rectype$recdata";
@@ -473,6 +470,11 @@ sub new {
         last;
       }
       last if $rectype eq 'N' && $recdata eq "";
+      if (!defined($rectype)) {
+         $ErrorFound = 1;
+      #print STDERR "RECTYPER ERROR: $rectype, $recdata\n";
+         last;
+      }
       if ($rectype eq 'p') {
 	 # //Glenn 2007-01-16
 	 # Handle p records (GOTO like in-place edit thing) by reading
@@ -487,44 +489,40 @@ sub new {
          $pRecordsFound = 1; # If we find a p record we need remember this to handle body ...
 	 next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	 if ($recdata+0 > $MaxpRecPos) {
-	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	    $ErrorFound = 1;
 	    last;
 	 }
-	 push @npos, $recdata+0; # save jumpto position, for loop detection.
 	 if ($OriginalPos == 0) {
 	    # Jump to after E record and commence
 	    $OriginalPos = tell $RQf;
 	    seek $RQf, $recdata+0, 0; # jump. we should check this works.
-#	    next;
 	 } else {
-	   # We're at the return point, or moving even furtehr away...
+	   # We're at the return point, or moving even further away...
 	   if ($recdata+0 < $OriginalPos) {
-	      # Error exit, carp or whatever. Probably should do the
-	      # nice exit Jules does above. For now, warn and use
-	      # OriginalPos
-	      MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	      seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	      $OriginalPos = 0; # We're home again... Reset to prepare for
-			        # any new warps
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	     #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0;
 	      @npos = ();
+	     $ErrorFound = 1;
+	     last;
 	   } else {
 	     seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-             if ($recdata+0 < $MsgContSize+$DataOffset) {
-	     #20090216 if ($recdata+0 < $npos[$#npos-1]) {
-	        if ($recdata+0 >= $npos[0]) {
-        	   MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+            foreach $_ (@npos) {
+              if ($_+0 eq $recdata+0) {
 		   $ErrorFound = 1;
-	           $OriginalPos = 0; # Reset to not fool next segment loop...
-	           @npos = ();
 		   last;
 	        }
-	        $OriginalPos = 0; # We're home again... Reset to prepare for
-		                  # any new warps
+	    }
+            if ($ErrorFound) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $OriginalPos = 0; # Reset to not fool next segment loop
 	        @npos = ();
+		  last;
 	     }
 	   }
 	 }
+	 push @npos, $recdata+0; # save jumpto position, for loop detection.
 	 next; # done, don't add a spurious "converted p to N" record.
       }
       next if ($rectype eq 'w'); # Skip deleted (w) records ... Else they will be transformed to Normal (N) records.
@@ -591,8 +589,8 @@ sub new {
 # Inelegant, but working. Instead of an efficient seek, we spinn through to
 # after X record. Unless we don't have a body to spin through. Also skip
 # the spin if we don't have any p records already (to not punish the normal
-# case).
-    if (!$message->{nobody} && $pRecordsFound) {
+# case). Don't spin if error found either.
+    if (!$message->{nobody} && $pRecordsFound && !$ErrorFound) {
       while(($rectype, $recdata) = ReadRecord($RQf)) {
         #print STDERR "Metadata type $rectype data \"$recdata\"\n";
         if (!defined($rectype) or $rectype eq 'X') {
@@ -612,11 +610,10 @@ sub new {
 	  # check that and act accordingly..
 	  next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	  if ($recdata+0 > $MaxpRecPos) {
-	      MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	      MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	      $ErrorFound = 1;
 	      last;
 	  }
-	  push @npos, $recdata+0; # save jumpto position, for loop detection.
 	  if ($OriginalPos == 0) {
 	     # Jump to after E record and commence
 	     $OriginalPos = tell $RQf;
@@ -625,30 +622,29 @@ sub new {
 	  } else {
 	    # We're at the return point, or moving even furtehr away...
 	    if ($recdata+0 < $OriginalPos) {
-	       # Error exit, carp or whatever. Probably should do the
-	       # nice exit Jules does above. For now, warn and use
-	       # OriginalPos
-	       MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	       seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	       $OriginalPos = 0; # We're home again... Reset to prepare for
-			         # any mew warps
+	       MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	       #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	       $OriginalPos = 0;
 	       @npos = ();
+	       $ErrorFound = 1;
+	       last;
 	    } else {
 	      seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-              if ($recdata+0 < $MsgContSize+$DataOffset) {
-	      #20090216 if ($recdata+0 < $npos[$#npos-1]) {
-	         if ($recdata+0 >= $npos[0]) {
-		    MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+              foreach $_ (@npos) {
+                if ($_+0 eq $recdata+0) {
 		    $ErrorFound = 1;
-		    $OriginalPos = 0; # Reset to not fool next segment loop...
 		    last;
 	         }
-	         $OriginalPos = 0; # We're home again... Reset to prepare for
-		                   # any mew warps
+	      }
+              if ($ErrorFound) {
+		 MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		 $OriginalPos = 0; # Reset to not fool next segment loop
 	         @npos = ();
+		 last;
 	      }
 	    }
 	  }
+	  push @npos, $recdata+0; # save jumpto position, for loop detection.
         }
       }
       #print STDERR "\n\nErrorFound=$ErrorFound and rectype=$rectype\n\n";
@@ -729,47 +725,61 @@ sub new {
 	# safe than sorry.
 	next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
 	if ($recdata+0 > $MaxpRecPos) {
-	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file (for now).");
 	    $ErrorFound = 1;
 	    last;
 	}
-	push @npos, $recdata+0; # save jumpto position, for loop detection.
 	if ($OriginalPos == 0) {
 	   # Jump to after E record and commence
 	   $OriginalPos = tell $RQf;
 	   seek $RQf, $recdata+0, 0; # jump. we should check this works.
 	   next;
 	} else {
-	  # We're at the return point, or moving even furtehr away...
+	  # We're at the return point, or moving even further away...
 	  if ($recdata+0 < $OriginalPos) {
-	     # Error exit, carp or whatever. Probably should do the
-	     # nice exit Jules does above. For now, warn and use
-	     # OriginalPos
-	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
-	     seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
-	     $OriginalPos = 0; # We're home again... Reset to prepare for
-			       # any mew warps
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, this cannot be! Aborting this file.");
+	     #seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0;
 	     @npos = ();
+	     $ErrorFound = 1;
+	     last;
 	  } else {
 	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
-	    if ($recdata+0 < $npos[$#npos-1]) {
-	       if ($recdata+0 >= $npos[0]) {
-		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+            foreach $_ (@npos) {
+              if ($_+0 eq $recdata+0) {
 		  $ErrorFound = 1;
-		  # no need to reset $OriginalPos since this is the last segment.
 		  last;
 	       }
-	       $OriginalPos = 0; # We're home again... Reset to prepare for
-		                 # any mew warps
+	    }
+            if ($ErrorFound) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $OriginalPos = 0; # Reset to not fool next segment loop
 	       @npos = ();
+		  last;
 	    }
 	  }
 	}
+	push @npos, $recdata+0; # save jumpto position, for loop detection.
       } else {
         # Some other record type. Just store it and move on.
         push @{$message->{metadata}}, "$rectype$recdata";
       }
     }
+    #print STDERR "\n\nErrorFound=$ErrorFound and rectype=$rectype\n\n";
+    # Found errors above, means the file isn't complete... remove it from the batch and return immediately.
+    if ($ErrorFound) {
+      #MailScanner::Log::WarnLog("No end-of-message record found in %s, " .
+      #                          "retrying", $message->{id});
+      # JKF 5/12/2005 This could fail with an unblessed reference error
+      # JKF 5/12/2005 so do it by hand.
+      # JKF 5/12/2005 $message->DropFromBatch();
+      $message->{deleted} = 1;
+      $message->{gonefromdisk} = 1; # Don't try to delete the original
+      $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+      #20090421 $message->{abandoned} = 1; # JKF 20090301 This message was scrapped
+
+      return 0;
+    }
       
     # Remove all the duplicates from ->{to}
     my %uniqueto;
@@ -886,7 +896,7 @@ sub new {
     $len = 0;
     $shift = 0;
     while (1) {
-      read $fh, $len_byte, 1;
+      read $fh, $len_byte, 1 or return (undef, undef);
       $len_byte = ord $len_byte;
       if ($shift >= 39) {
         MailScanner::Log::WarnLog("Postfix record too long in ReadRecord()");
@@ -1842,7 +1852,8 @@ sub FindHashDirDepth {
         #print STDERR "Added $id to batch\n";
         $batchempty = 0;
 
-        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+        if (MailScanner::Config::Value("scanmail", $newmessage) =~ /[12]/ ||
+            MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
             MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
           $newmessage->NeedsScanning(1);
           $DirtyMsgs++;
