@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm 4760 2009-04-21 21:03:34Z sysjkf $
+#   $Id: SA.pm 5018 2010-05-26 16:13:59Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use Compress::Zlib;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4760 $, 10;
+$VERSION = substr q$Revision: 5018 $, 10;
 
 # Attributes are
 #
@@ -138,6 +138,7 @@ sub initialise {
 
   unless (MailScanner::Config::IsSimpleValue('usespamassassin') &&
           !MailScanner::Config::Value('usespamassassin')) {
+    $settings{post_config_text} = MailScanner::ConfigSQL::ReturnSpamAssassinConfig();
     $settings{dont_copy_prefs} = 1; # Removes need for home directory
     # This file is now read directly by SpamAssassin's normal startup code.
     #$prefs = MailScanner::Config::Value('spamassassinprefsfile');
@@ -566,6 +567,15 @@ sub Checks {
   push(@WholeMessage, $fromheader . ': ' . $message->{from} . "\n")
     if $fromheader;
 
+  # Add the spamvirusreport to the input to SA.
+  # The header name should be documented in the MailScanner.conf docs.
+  my $svheader = MailScanner::Config::Value('spamvirusheader', $message);
+  if ($svheader && $message->{spamvirusreport}) {
+    $svheader =~ s/:$//;
+    push(@WholeMessage, $svheader . ': ' . $message->{spamvirusreport} . "\n");
+    #print STDERR "Added $svheader: " . $message->{spamvirusreport} . "\n";
+  }
+
   #push(@WholeMessage, $global::MS->{mta}->OriginalMsgHeaders($message, "\n"));
   push(@WholeMessage, @SAheaders);
   #print STDERR "Headers are : " . join(', ', @WholeMessage) . "\n";
