@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Sendmail.pm 4813 2009-05-08 10:48:31Z sysjkf $
+#   $Id: Sendmail.pm 5044 2010-07-31 16:58:58Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ use DBI;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4813 $, 10;
+$VERSION = substr q$Revision: 5044 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -1126,7 +1126,8 @@ my($sed) = "/bin/sed";
         $newmessage->{mtime} = $mtime;
         $batchempty = 0;
 
-        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+        if (MailScanner::Config::Value("scanmail", $newmessage) =~ /[12]/ ||
+            MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
             MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
           $newmessage->NeedsScanning(1);
           $DirtyMsgs++;
