@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepOther.pm 5011 2010-04-22 16:12:43Z sysjkf $
+#   $Id: SweepOther.pm 5065 2010-11-13 15:14:59Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 5011 $, 10;
+$VERSION = substr q$Revision: 5065 $, 10;
 
 # Attributes are
 #
@@ -110,6 +110,7 @@ sub ScanBatch {
       next if $message->{deleted};  # Don't do deleted messages!
 
       # Check the message if *any* recipient wants dangerous content scanning
+      next if $message->{scanvirusonly};
       next unless MailScanner::Config::Value('dangerscan', $message) =~ /1/;
       my @headers = <$headerfh>;
 
@@ -160,6 +161,7 @@ sub ScanBatch {
       next if $message->{deleted};  # Don't do deleted messages!
 
       # Check the message if *any* recipient wants dangerous content scanning
+      next if $message->{scanvirusonly};
       next unless MailScanner::Config::Value('dangerscan', $message) =~ /1/;
 
       # Find the name of the TNEF winmail.dat if it exists
@@ -173,6 +175,13 @@ sub ScanBatch {
       # attachments that can from archives.
       $ArchivesAre = $message->{archivesare};
 
+      # Read the filename renaming pattern for this message.
+      # This is used for "rename" rules in filename and filetype rules.conf.
+      # And make it a filename suffix if they didn't read the instructions!
+      my $RenamePattern = MailScanner::Config::Value('defaultrenamepattern', $message);
+      $RenamePattern = '__FILENAME__' . $RenamePattern
+        unless $RenamePattern =~ /__FILENAME__/i;
+
       # Set up patterns for simple filename real rules files
       my($allowpatterns, $denypatterns, $allowexists, $denyexists,
          @allowpatterns, @denypatterns, $megaallow,   $megadeny);
@@ -325,7 +334,41 @@ sub ScanBatch {
           $MatchFound = 1;
 
           #print STDERR "\"$attach\" matched \"$regexp\" or \"$safename\" did\n";
-          if ($allowdeny =~ /^deny[^@]*$/i) {
+          if ($allowdeny =~ /^rename/i) {
+            # It's a rename rule, not really a rejection or acceptance.
+            MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
+                                      $logtext, $id, $attach);
+            my $entity = $message->{file2entity}{$attach};
+            if ($entity && $entity->head) {
+              my $newname = '';
+              if ($allowdeny =~ s/^rename //i) {
+                # They have supplied the replacement text, now in $allowdeny
+                $newname = $attach;
+                # This lets them put things like $1 and $2 in the RHS.
+                eval '$newname =~ s/'.$regexp.'/'.$allowdeny.'/i';
+                #$newname =~ s/$regexp/$allowdeny/is;
+              } else {
+                # No replacement text, so use the default rename pattern
+                # if it was set in MailScanner.conf.
+                if ($RenamePattern) {
+                  $newname = $RenamePattern;
+                  $newname =~ s/__FILENAME__/$attach/i;
+                }
+              }
+              if ($newname ne '') {
+                MailScanner::Log::InfoLog("Filename Checks: %s renamed %s to %s",
+                                          $id, $attach, $newname);
+                #print STDERR "Renaming $attach to $newname\n";
+                # These are the only 2 places the user's filename has to change
+                $entity->head->mime_attr("content-disposition.filename",
+                                         $newname);
+                $entity->head->mime_attr("content-type.name",
+                                         $newname);
+                # Do no more than this, we aren't generating loads of reports
+                $message->{bodymodified} = 1;
+              }
+            }
+          } elsif ($allowdeny =~ /^deny[^@]*$/i) {
             # It's a deny or deny+delete, not an email address.
             # It's a rejection rule, so log the error.
             MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
@@ -471,7 +514,9 @@ sub CheckFileContentTypes {
 
   # If we are not using "file -i" at all, then just short-circuit all
   # of this, and check file command output only.
-  unless ($MailScanner::Config::UsingFileICommand) {
+  unless ($MailScanner::Config::UsingFileICommand ||
+           MailScanner::Config::Value('denyfilemimetypes') ||
+           MailScanner::Config::Value('adenyfilemimetypes')) {
     $Counter = CheckFileTypesRules($batch, \%FileTypes, undef);
     return $Counter;
   }
@@ -568,7 +613,7 @@ sub CheckFileTypesRules {
      @Aallowpatterns, @Adenypatterns, $Amegaallow,   $Amegadeny);
   my($Nallowpatterns, $Ndenypatterns, $Nallowexists, $Ndenyexists,
      @Nallowpatterns, @Ndenypatterns, $Nmegaallow,   $Nmegadeny);
-  my($LogTypes);
+  my($LogTypes, $RenamePattern);
   my $counter = 0;
 
   # winmail.dat will always be a normal attachment
@@ -580,6 +625,7 @@ sub CheckFileTypesRules {
     $message = $batch->{messages}{$id};
 
     # Check the message if *any* recipient wants dangerous content scanning
+    next if $message->{scanvirusonly};
     next unless MailScanner::Config::Value('dangerscan', $message) =~ /1/;
 
     # Optimisation: Don't do this if we're still on the same message.
@@ -592,6 +638,13 @@ sub CheckFileTypesRules {
       $LogTypes = MailScanner::Config::Value('logpermittedfiletypes',
                                                 $message);
 
+      # Read the filename renaming pattern for this message.
+      # This is used for "rename" rules in filename and filetype rules.conf.
+      # And make it a filename suffix if they didn't read the instructions!
+      $RenamePattern = MailScanner::Config::Value('defaultrenamepattern', $message);
+      $RenamePattern = '__FILENAME__' . $RenamePattern
+        unless $RenamePattern =~ /__FILENAME__/i;
+
       # Set up patterns for simple filename real rules files
       $Aallowpatterns = MailScanner::Config::Value('aallowfiletypes', $message);
       $Adenypatterns  = MailScanner::Config::Value('adenyfiletypes', $message);
@@ -700,7 +753,29 @@ sub CheckFileTypesRules {
         #print STDERR "Filetype match: $allowdeny $regexp $logtext $usertext\n";
 
         $MatchFound = 1;
-        if ($allowdeny =~ /^deny[^@]*$/i) {
+
+        if ($allowdeny =~ /^rename/i) {
+          # It's a rename rule, not really a rejection or acceptance.
+          MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
+                                    $logtext, $id, $attach);
+          my $entity = $message->{file2entity}{$attach};
+          #print STDERR "entity = $entity\n";
+          if ($entity && $entity->head && $RenamePattern) {
+            my $newname = $RenamePattern;
+            $newname =~ s/__FILENAME__/$attach/i;
+
+            MailScanner::Log::InfoLog("Filetype Checks: %s renamed %s to %s",
+                                      $id, $attach, $newname);
+            #print STDERR "Renaming $attach to $newname\n";
+            # These are the only 2 places the user's filename has to change
+            $entity->head->mime_attr("content-disposition.filename",
+                                     $newname);
+            $entity->head->mime_attr("content-type.name",
+                                     $newname);
+            # Do no more than this, we aren't generating loads of reports
+            $message->{bodymodified} = 1;
+          }
+        } elsif ($allowdeny =~ /^deny[^@]*$/i) {
           # It's a deny or deny+delete, not an email address
           # It's a rejection rule, so log the error.
           MailScanner::Log::InfoLog("Filetype Checks: %s (%s %s)",
@@ -759,6 +834,7 @@ sub CheckFileTypesRules {
     if ($lastid ne $id) {
       $lastid = $id;
       # Check the message if *any* recipient wants dangerous content scanning
+      next if $message->{scanvirusonly};
       next unless MailScanner::Config::Value('dangerscan', $message) =~ /1/;
 
       $LogTypes = MailScanner::Config::Value('logpermittedfilemimetypes',
