@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ZMailer.pm 4522 2008-08-20 15:19:23Z sysjkf $
+#   $Id: ZMailer.pm 5044 2010-07-31 16:58:58Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use Encode;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4522 $, 10;
+$VERSION = substr q$Revision: 5044 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -239,7 +239,8 @@ sub ReadQf {
   my($ip, $TOline, $RcvdFromLine);
   my($Line, $Flags);
   my($TOFound, $FROMFound, $IPFound);
-  my($read1strcvd, $ipfromheader);
+  my(@rcvdiplist);
+  #my($read1strcvd, $ipfromheader);
 
   #$message->{store}->print();
 
@@ -329,16 +330,22 @@ sub ReadQf {
       if ($Line =~ /^Subject:\s*(\S.*)?$/i) {
         $message->{subject} = $1;
       }
-      if ($Line =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
-        unless ($read1strcvd) {
-          $ipfromheader = $1;
-          $read1strcvd = 1;
-        }
-      } elsif ($Line =~ /^Received: .+\[([\dabcdef.:]+)\]/i) {
-        unless ($read1strcvd) {
-          $ipfromheader = $1;
-          $read1strcvd = 1;
+      if ($Line =~ /^Received:/i) {
+        my $rcvdip = '127.0.0.1';
+        if ($Line =~ /^Received: .+?\[(\d+\.\d+\.\d+\.\d+)\]/i) {
+          $rcvdip = $1;
+          #unless ($read1strcvd) {
+          #  $ipfromheader = $1;
+          #  $read1strcvd = 1;
+          #}
+        } elsif ($Line =~ /^Received: .+?\[([\dabcdef.:]+)\]/i) {
+          $rcvdip = $1;
+          #unless ($read1strcvd) {
+          #  $ipfromheader = $1;
+          #  $read1strcvd = 1;
+          #}
         }
+        push @rcvdiplist, $rcvdip;
       }
       # JKF Testing code, remove before production release
       #if ($Line =~ /^X-ClientIP:/i) {
@@ -350,8 +357,12 @@ sub ReadQf {
   }
 
   # If we were told to get the IP address from the header, and it was there...
-  $message->{clientip} = $ipfromheader
-    if $getipfromheader && $read1strcvd && $ipfromheader ne "";
+  $getipfromheader = @rcvdiplist if $getipfromheader>@rcvdiplist;
+  # If they wanted the 2nd Received from address, give'em element 1 of list
+  $message->{clientip} = $rcvdiplist[$getipfromheader-1] if
+    $getipfromheader>0;
+  #$message->{clientip} = $ipfromheader
+  #  if $getipfromheader && $read1strcvd && $ipfromheader ne "";
 
   # Decode ISO subject lines into UTF8
   # Needed for UTF8 support in MailWatch 2.0
@@ -434,18 +445,30 @@ sub AddHeader {
   unless( grep( /^env-end$|^env-eof$/i, @{$message->{metadata}} ) ) {
     push @{$message->{metadata}}, 'env-end';
   }
-  push @{$message->{metadata}}, "$newkey $newvalue";
+  # DKIM: May just need to unshift instead of push?
+  if ($message->{newheadersattop}) {
+    unshift @{$message->{metadata}}, "$newkey $newvalue";
+  } else {
+    push @{$message->{metadata}}, "$newkey $newvalue";
+  }
 }
 # REVISO LEOH
 sub DeleteHeader {
   my($this, $message, $key) = @_;
 
+  my $usingregexp = ($key =~ s/^\/(.*)\/$/$1/)?1:0;
+
+  # Add a colon if they forgot it.
+  $key .= ':' unless $usingregexp || $key =~ /:$/;
+  # If it's not a regexp, then anchor it and sanitise it.
+  $key = '^' . quotemeta($key) unless $usingregexp;
+
   my($linenum);
   for ($linenum=0; $linenum<@{$message->{metadata}}; $linenum++) {
     last if $message->{metadata}[$linenum] =~ /^env-end$|^env-eof$/i;
   }
   for ($linenum++; $linenum<@{$message->{metadata}}; $linenum++) {
-    next unless $message->{metadata}[$linenum] =~ /^$key/i;
+    next unless $message->{metadata}[$linenum] =~ /$key/i;
     # Have found the right line
     splice(@{$message->{metadata}}, $linenum, 1);
     # Now delete the continuation lines
@@ -482,7 +505,8 @@ sub UniqHeader {
 sub ReplaceHeader {
   my($this, $message, $key, $newvalue) = @_;
 
-  $this->DeleteHeader($message, $key);
+  # DKIM: Don't do DeleteHeeader if adding at top of headers
+  $this->DeleteHeader($message, $key) unless $message->{dkimfriendly};
   $this->AddHeader($message, $key, $newvalue);
 }
 
@@ -879,6 +903,7 @@ sub CreateBatch {
     }
 
     $batchempty = 1;
+    my $maxattempts = MailScanner::Config::Value('procdbattempts');
 
     # Keep going until end of dir or have reached every imposed limit. This
     # now processes the files oldest first to make for fairer queue cleanups.
@@ -911,16 +936,80 @@ sub CreateBatch {
       # Lock and read the qf file. Skip this message if the lock fails.
       $newmessage = MailScanner::Message->new($id, $queuedirname,
                                               $getipfromheader);
-      if ($newmessage eq 'INVALID') {
+      if ($newmessage && $newmessage->{INVALID}) {
+      #if ($newmessage eq 'INVALID') {
         $invalidfiles .= "$id ";
+        undef $newmessage;
         next;
       }
       next unless $newmessage;
+
+      # JKF 20090301 Skip this message if $id has been scanned
+      # too many times.
+      # JKF 20090301 Read the number of times this message id has
+      # been processed. If over the limit, then ignore it.
+      # JKF 20090301 Should ideally make it in a batch of its own,
+      # and try that once or twice, *then* skip it. But let's try
+      # a simple version first.
+      # Just do a "next" if we want to skip the message.
+      if ($maxattempts) {
+        my $nexttime = time + 120 + int(rand(240)); # 4 +- 2 minutes
+        #my $nexttime = time + 10 + int(rand(20)); # 4 +- 2 minutes
+        my @attempts = $MailScanner::ProcDBH->selectrow_array(
+                         $MailScanner::SthSelectRows, undef, $id);
+        #my @attempts = $MailScanner::ProcDBH->selectrow_array(
+        #"SELECT id,count,nexttime FROM processing WHERE (id='" . $id . "')");
+        #print STDERR "id       = \"$attempts[0]\"\n";
+        #print STDERR "count    = \"$attempts[1]\"\n";
+        #print STDERR "nexttime = \"$attempts[2]\"\n";
+        if (@attempts && $attempts[1]>=$maxattempts) {
+          MailScanner::Log::WarnLog("Warning: skipping message %s as it has been attempted too many times", $id);
+          # JKF 20090301 next;
+          # Instead of just skipping it, quarantine it and notify
+          # the local postmaster.
+          $newmessage->QuarantineDOS();
+          #print STDERR "Moving $attempts[0], $attempts[1], $attempts[2] into achive table\n";
+          $MailScanner::SthDeleteId->execute($id);
+          #$MailScanner::ProcDBH->do(
+          #  "DELETE FROM processing WHERE id='" . $id . "'");
+          $MailScanner::SthInsertArchive->execute($attempts[0], $attempts[1],
+                                             $attempts[2]);
+          #$MailScanner::ProcDBH->do(
+          #  "INSERT INTO archive (id, count, nexttime) " .
+          #  "VALUES ('$attempts[0]', $attempts[1], $attempts[2])");
+        } elsif (defined $attempts[1]) {
+          # We have tried this message before
+          if (time>=$attempts[2]) {
+            # Time for next attempt has arrived
+            $MailScanner::SthIncrementId->execute($nexttime, $id);
+            #$MailScanner::ProcDBH->do(
+            #  "UPDATE processing SET count=count+1, nexttime=$nexttime " .
+            #  " WHERE id='" . $id . "'");
+            MailScanner::Log::InfoLog(
+              "Making attempt %d at processing message %s",
+              $attempts[1]+1, $id);
+            #print STDERR "Incremented $id\n";
+          } else {
+            # Not time for next attempt yet, so ignore the message
+            $newmessage->DropFromBatch();
+            next;
+          }
+        } else {
+          # We have never seen this message before
+          $MailScanner::SthInsertProc->execute($id, 1, $nexttime);
+          #$MailScanner::ProcDBH->do(
+          #  "INSERT INTO processing (id, count, nexttime) " .
+          #  "VALUES ('" . $id . "', 1, $nexttime)");
+          #print STDERR "Inserted $id\n";
+        }
+      }
+
       $batch->{messages}{"$id"} = $newmessage;
       $newmessage->{mtime} = $mtime;
       $batchempty = 0;
 
-      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+      if (MailScanner::Config::Value("scanmail", $newmessage) =~ /[12]/ ||
+          MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
           MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
         $newmessage->NeedsScanning(1);
         $DirtyMsgs++;
